"""
globals
"""


__version__ = "0.0.5"


LIBRARY = "pybugsnag"
API_URL = "https://api.bugsnag.com/"


TEST_API_URL = "https://private-anon-3633b611b0-bugsnagapiv2.apiary-mock.com/"
TEST_TOKEN = "access_token"
